# cookie

Блок предоставляет объект, содержащий набор методов для работы с куки браузера (JS-свойство `document.cookie`).

## Обзор

### Свойства и методы объекта

| Имя | Возвращаемое значение | Описание |
| -------- | --- | -------- |
| <a href="#fields-get">get</a>(<code>{String} name</code>) | <code>{String}</code>&#124;<code>{null}</code> | Cлужит для получения значения, хранящегося в куки браузера. |
| <a href="#fields-set">set</a>(<code>{String} name</code>, <br><code>{String} val</code>, <br><code>[{Object} options]</code>) | <code>{String}</code> | Cлужит для записи куки с заданным именем.|

### Публичные технологии блока

Блок реализован в технологиях:

* `js`

## Описание

<a name="fields"></a>
### Свойства и методы объекта

<a name="fields-get"></a>
#### Метод `get`

Метод служит для получения значения, хранящегося в куки, для имени переданного аргументом.

Принимаемые аргументы:

* `name` `{String}` – имя куки. Обязательный аргумент.

Возвращаемое значение:

* `{String}` в случае если куки с заданным именем было установлено. Значение автоматически декодируется с помощью [decodeURIComponent](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/decodeURIComponent).
* `null` если куки с заданным именем отсутствует.

```js
modules.require('cookie', function(cookie) {

cookie.set('mycookie', 'foobar');
console.log(cookie.get('mycookie')); // 'foobar'
console.log(cookie.get('foo')); // null
    
});
```

<a name="fields-set"></a>
#### Метод `set`

Метод служит для записи куки с заданным именем. Помимо имени и значения, методу можно передать хеш с дополнительными параметрами куки.

Принимаемые аргументы:

* `name` `{String}` – имя куки. Обязательный аргумент.
* `val` `{String}` | `{null}` – строка со значением, ассоциируемым с именем. При установке в качестве значения `null` куки будет удалено. Обязательный аргумент.
* [`options` `{Object}`] – хеш с дополнительными параметрами. В хеше допустимы следующие свойства:
    * `expires` `{Number}` – срок жизни куки в сутках. При отрицательном значении куки будет удалено. Альтернативно, можно передать в качестве значения сформированный объект даты (`new Date()`).
    * `path` `{String}` – путь от корня домена внутри которого будет доступно куки.
    * `domain` `{String}` – домен в явном виде. По умолчанию – текущий.
    * `secure` `{Boolean}` – логический флаг, указывающий на необходимость принудительного использования с куки шифрованного соединения SSL. По умолчанию `false`.

Возвращает объект `this`.

```js
modules.require('cookie', function(cookie) {

cookie.set('mycookie', 'foobar', {
    expires : 1, // срок жизни одни сутки
    path : '/', // доступно для всех страниц домена
    secure : true // передавать куки только по SSL
});
console.log(cookie.get('mycookie')); // 'foobar'

cookie.set('mycookie', null }); // удаляем куки
console.log(cookie.get('mycookie')); // null

});
```
