# querystring

Блок предоставляет объект, содержащий набор методов для работы со строкой запроса формата URI.

## Обзор

### Свойства и методы объекта

| Имя | Возвращаемое значение | Описание |
| --- | --------------------- | -------- |
| <a href="#fields-parse">parse</a>(<code>{String} str</code>) | <code>{Object}</code> | Формирует объект на основании параметров запроса адресной строки. |
| <a href="#fields-stringify">stringify</a>(<code>{Object} obj</code>) | <code>{String}</code> | Формирует строку запроса на основании свойств объекта. |

### Элементы блока

| Элемент | Способы использования | Описание |
| --------| --------------------- | -------- |
| <a href="#elems-uri">uri</a> | <code>JS</code> | Элемент предназначен для декодирования строки из формата URI. |

#### Свойства и методы объекта

| Элемент | Имя | Возвращаемое значение | Описание |
| ------- | --- | --------------------- | -------- |
| <a href="#elems-uri">uri</a> | <a href="#elems-uri-fields-decodeURI">decodeURI</a>(<code>{String} str</code>) | <code>{String}</code> | Служит для декодирования URI. |
| | <a href="#elems-uri-fields-decodeURIComponent">decodeURIComponent</a>(<code>{String} str</code>) | <code>{String}</code> | Служит для декодирования URI компонента. |

### Публичные технологии блока

Блок реализован в технологиях:

* `vanilla.js`

<a name="fields"></a>
### Свойства и методы объекта

<a name="fields-parse"></a>
#### Метод `parse` 

Формирует объект на основании строки параметров в формате URI. 

Принимаемые аргументы: 

* `str` `{String}` – строка с параметрами в виде пар ключ-значение. Ключ отделяется от значения символом `=`. Пары разделяются символом `&`. В процессе обработки, ключи и значения декодируются из формата URI. Обязательный аргумент.

Возвращаемое значение: `{Object}`. Объект, сформированный на основании параметров адресной строки.

<a name="fields-stringify"></a>
#### Метод `stringify`

Формирует строку запроса в формате URI на основании объекта. 

Принимаемые аргументы: 

`obj` `{Object}` – объект, на основании которого формируется строка. Обязательный аргумент.

Возвращаемое значение: `{String}`. Имена свойств в строке отделяются от значений символом `=`, пары разделяются символом `&`. 

<a name="elems"></a>
### Элементы блока

<a name="elems-uri"></a>
#### Элемент `uri`

Элемент предоставляет объект, содержащий набор методов для декодирования строки из формата URI. 

<a name="elems-name-fields"></a>
### Свойства и методы объекта

Оба метода являются оберткой над соответствующими стандартными методами JavaScript `decodeURI` и `decodeURIComponent`. 

В ходе работы методы проверяют соответствие формата переданной строки UTF-8. При несоответствии генерируется ошибка.

<a name="elems-uri-fields-decodeURI"></a>
#### Метод `decodeURI`

Служит для декодирования URI. Метод идентичен стандартному методу JavaScript [decodeURI](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/decodeURI), но поддерживает кириллическую кодировку `CP-1251`.

Принимаемые аргументы: 

* `str` `{String}` – строка с последовательностями экранирования. Обязательный аргумент.

Возвращаемое значение: `{String}`. В случае если последовательности экранирования в строке не найдены метод возвращают ее без изменений.

```js
modules.require('querystring__uri', function(uri){
    uri.decodeURI("https://developer.mozilla.org/ru/docs/JavaScript_%D1%88%D0%B5%D0%BB%D0%BB%D1%8B");
    // "https://developer.mozilla.org/ru/docs/JavaScript_шеллы"
})
```


<a name="elems-uri-fields-decodeURIComponent"></a>
#### Метод `decodeURIComponent`

Служит для декодирование компонента URI. Метод идентичен стандартному методу JavaScript [decodeURIComponent](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/decodeURIComponent), но поддерживает кириллическую кодировку `CP-1251`.

Принимаемые аргументы: 

* `str` `{String}` – строка с последовательностями экранирования. Обязательный аргумент.

Возвращаемое значение: `{String}`. В случае если последовательности экранирования в строке не найдены метод возвращают ее без изменений.

```js
modules.require('querystring__uri', function(uri){
    uri.decodeURIComponent("JavaScript_%D1%88%D0%B5%D0%BB%D0%BB%D1%8B");
    // "JavaScript_шеллы"
})
```
