<a name="context"></a>
## Контекст

**Методы экземпляра блока** исполняются в контексте JS-объекта экземпляра блока. Ключевое слово `this` в методах экземпляра блока ссылается на JS-объект **экземпляра блока**.

**Статические методы** исполняются в контексте JS-объекта, соответствующего классу блока. Ключевое слово `this` в статических методах блока ссылается на **класс блока**.

***

**NB** При разработке блоков с использованием `i-bem.js` внутренним методам блока, не предназначенным для использования извне, принято давать имена, начинающиеся с символа подчеркивания. Например, `_onClick`.

***

### Свойства экземпляра блока

#### С DOM-представлением

* `params` – хэш параметров, переданных экземпляру блока при инициализации.
* `domElem` – объект jQuery, содержащий ссылки на DOM-элементы, к которым [привязан][html] блок.

#### Без DOM-представления

* `params` – хэш параметров, переданных экземпляру блока при инициализации.

<a name="spec-fields"></a>
#### Вспомогательные свойства

Экземпляр блока предоставляет набор вспомогательных свойств: 

* `__self` – для доступа к статическим свойствам и методам блока из его экземпляра.

**Пример:** Вызов статического метода `staticMethod` в методе `onEvent` экземпляра блока `my-block`.

```js
BEMDOM.decl('my-block', {
    onEvent : function() {
        this.__self.staticMethod(); // вызов статического метода
        this.doMore();
    }
}, {
    staticMethod : function() { /* ... */ }; // определение статического метода
});
```


* `__base` – для вызова реализации одноименного метода из базового класса, от которого наследуется данный (`super call`).

**Пример:** вызов базовой реализации метода `_onClick` базового класса `button`.

```js
BEMDOM.decl({ block : 'my-button', baseBlock : 'button' }, {
    _onClick : function() {
        this.__base(); // вызываем базовый _onClick
        this.doMore();
    }
});
```


Вспомогательные свойства предоставляются модулем [inherit](../../common.blocks/inherit/inherit.ru.md), реализующим механизм наследования в `bem-core`.

### Статические свойства блока

<a name="spec-fields-static"></a>
#### Вспомогательные свойства

В декларации статических методов блока доступны вспомогательные свойства:

* `__base` – для вызова реализации одноименного метода из базового класса, от которого наследуется данный (`super call`).

```js
BEMDOM.decl({ block : 'extra', baseBlock : 'my-block' }, 
    { /* ... */ },
    { 
        staticMethod: function() {
            this.__base();
            this.doMore();
        }
    }
);
```


### Статические свойства модуля BEMDOM

* `scope` – корневой элемент обрабатываемого DOM-дерева. Позволяет выполнять несколько разных версий `i-bem.js` в одном рантайме. По умолчанию содержит ссылку на jQuery объект `body`.
* `doc` – ссылка на jQuery объект `document`.
* `win` – ссылка на jQuery объект `window`.


[html]: ./i-bem-js-html-binding.ru.md
