<a name="init"></a>
## Инициализация

Инициализация блока — это создание в памяти браузера JS-объекта,
соответствующего экземпляру блока. Инициализация экземпляров блоков выполняется
методом `init()` модуля `i-bem__dom` на заданном фрагменте DOM-дерева.

Каждому экземпляру блока можно приписать три состояния:

* экземпляр блока не инициализирован (JS-объект не создан);
* экземпляр блока инициализирован (JS-объект создан в памяти браузера);
* экземпляр блока уничтожен (удалены все ссылки на JS-объект экземпляра
блока, и он может быть удален сборщиком мусора).

В `i-bem.js` эти состояния экземпляра блока описываются с помощью служебного
модификатора `js`.

* До инициализации экземпляр блока не имеет модификатора `js`.

```html
<div class="my-block i-bem" data-bem="..." >...</div>
```


* В момент инициализации экземпляру блока устанавливается модификатор
`js` в значении `inited`.

```html
<div class="my-block i-bem my-block_js_inited" data-bem="...">...</div>
```


* Если в процессе работы удаляется фрагмент DOM-дерева (при помощи метода `destruct` модуля `i-bem__dom`), то вместе с ним удаляются экземпляры блоков, все HTML-элементы которых находятся в этом фрагменте. Перед удалением экземпляра блока модификатор `js` удаляется, чтобы выполнились [деструкторы экземпляра](#destruct) блока.

***

**NB** Если экземпляр блока был [привязан к нескольким HTML-элементам](i-bem-js-html-binding.ru.md#distrib-block), блок будет существовать, пока в HTML-дереве сохраняется хотя бы один элемент, с которым он связан.

***

Если на HTML-элементе размещено несколько экземпляров других блоков, то
инициализация одного из них (появление модификатора `js_inited`)
не влияет на инициализацию остальных.

**Пример:** На HTML-элементе инициализирован только экземпляр блока `my-block`.
Экземпляр блока `lazy-block` не инициализирован:

```html
<div class="my-block my-block_js_inited lazy-block i-bem"
    data-bem='{ "my-block": {}, "lazy-block": {} }' >
    ...
</div>
```


***

**NB** Наличие модификатора `js` позволяет писать разные CSS-стили для
блока в зависимости от того, инициализирован он или нет.

***

<a name="constructor"></a>
### Конструктор экземпляра блока

На изменение значений модификатора `js` можно назначать [триггеры](i-bem-js-states.ru.md#mods-api-trigger) так же, как и для любых других модификаторов блока.

Триггер на установку модификатора `js` в значение `inited` выполняется
при создании блока. Этот триггер можно считать **конструктором экземпляра блока**:

```js
onSetMod: {
    'js': {
        'inited': function() { /* ... */ } // конструктор экземпляра блока
    }
}
```


<a name="destruct"></a>
### Деструктор экземпляра блока

Моментом удаления блока является момент уничтожения всех ссылок на
JS-объект блока, после чего он может быть удален из памяти браузера
сборщиком мусора.

Триггер на удаление модификатора `js` (установку в пустое значение
`''`) выполняется перед удалением блока. Такой триггер можно считать
**деструктором экземпляра блока**.

```js
onSetMod: {
    'js': {
        '': function() { /* ... */ } // деструктор экземпляра блока
    }
}
```


<a name="init-wave"></a>
### Волны инициализации

Инициализация экземпляров блоков, присутствующих на странице, не
обязательно происходит одновременно. Блоки могут динамически добавляться в ходе
работы, инициализироваться по запросу или событию.
Инициализация очередной группы блоков называется **волной инициализации**.

Новая волна инициализации создается в следующих случаях:

* [Автоматическая инициализация блоков по событию `domReady`](#init-auto)
* [Инициализация блока по событию](#init-live) (ленивая инициализация)
* [Явный вызов инициализации блоков на указанном фрагменте DOM-дерева](#init-ajax)

<a name="init-auto"></a>
### Автоматическая инициализация

Фреймворк *i-bem.js* позволяет автоматически инициализировать блоки с DOM-представлением в момент наступления DOM-события `domReady`.

При автоматической инициализации в памяти браузера будут созданы JS-объекты для всех DOM-узлов, содержащих `i-bem` в атрибуте `class`. Инициализация выполняется функцией `init` модуля [i-bem__dom][].

Включить автоматическую инициализацию можно, указав блок `i-bem` с модификатором `init` в значении `auto` в файле зависимостей `.deps.js`.

**Пример файла** `.deps.js`:

```js
({
    shouldDeps: [
        {
            block: 'i-bem',
            elem: 'dom',
            mods: { 'init': 'auto' }
        }
    ]
})
```


Блок [page](../../common.blocks/page/page.ru.md) уже содержит в зависимостях `i-bem__dom_init_auto`, поэтому если он используется в проекте, не требуется ничего дополнительно подключать.

***

**NB** Блоки, для которых задана ленивая инициализация, не будут инициализированы автоматически.

***

<a name="init-live"></a>
### Инициализация по событию (ленивая инициализация)

Если на странице размещено много экземпляров блоков, автоматическая инициализация
всех блоков в момент загрузки страницы нежелательна, так как она
увеличивает время загрузки и объем памяти, затрачиваемой браузером.

Удобнее инициализировать JS-объекты только в тот момент, когда их функциональность потребуется
пользователю: например, по клику на блоке. Такая инициализация называется **ленивой** или **live-инициализацией**.

Для описания условий ленивой инициализации в декларации зарезервировано статическое свойство `live`. Свойство `live` может иметь значения следующих типов:

`Boolean`

* `true` — экземпляры блоков данного класса будут инициализированы только при попытке получить соответствующий экземпляр (см. раздел «[Взаимодействие блоков][interact]»).

```js
modules.define('my-block', ['i-bem__dom'], function(provide, BEMDOM) {

provide(BEMDOM.decl(this.name,
    {
        onSetMod: {
            'js': {
                'inited': function() { /* ... */ } // этот код будет выполняться
                                                   // при первом обращении к экземпляру блока
            }
        }
    },
    { live: true } // статические методы и свойства
));

});
```

* `false` — позволяет отменить ленивую инициализацию блоков, заданную на другом уровне переопределения.

`Function` – функция, выполняемая перед инициализацией **первого экземпляра** блока заданного класса. Если функция возвращает значение `false`, экземпляры блока будут инициализированы [автоматически](#init-auto).

```js
modules.define('my-block', ['i-bem__dom', 'ua'], function(provide, BEMDOM, ua) {

provide(BEMDOM.decl(this.name, {
    onSetMod : {
        'js' : {
            'inited' : function() {
                // выполнится при первом обращении к экземпляру блока
            }
        }
    }
}, {
    live : function() { // выполнится перед инициализацией первого экземпляра блока
        if(ua.msie && ua.version < 9) {
                          // отмена ленивой инициализации блока
            return false; // для старых версий Internet Explorer
        }
    }
}));

});
```


***

**NB** Ленивая инициализация может быть отменена для конкретного экземпляра блока. Для этого  нужно указать в [параметрах][params] HTML-элемента, к которому привязан экземпляр блока `data-bem='{"live": false}'`.

***

Чтобы инициализировать экземпляры блока по наступлению DOM-события или БЭМ-события, в теле функции следует выполнить подписку на [делегированные события](i-bem-js-events.ru.md#delegated-events) или воспользоваться [хелпером](#init-live-helpers).

**Пример:** Экземпляры блока `my-block` будут инициализированы по DOM-событию `click` на DOM-узле блока. По каждому DOM-событию `click` будет вызываться метод экземпляра блока `_onClick`:

```js
modules.define('my-block', ['i-bem__dom'], function(provide, BEMDOM) {

provide(BEMDOM.decl(this.name, {
    onSetMod: {
        'js': {
            'inited': function() { /* ... */ } // выполняется при первом DOM-событии "click"
        }
    },

    _onClick: function(e) { /* ... */ } // выполняется при каждом DOM-событии "click"
}, {
    live: function() {
        this.liveBindTo('click', function(e) {
            this._onClick(e);   // в момент клика будет создан экземпляр
                                // блока и вызван его метод _onClick
        });
    }
}));

});
```


***

**NB** Свойство `live` относится к статическим методам класса блока. Поэтому даже если оно задано в декларации блока с определенным модификатором, `live` будет применено ко всем блокам данного класса вне зависимости от модификаторов.

***

<a name="init-live-helpers"></a>
### Хелперы для инициализации по событию

Для упрощения инициализации по событию в контексте экземпляра блока зарезервирован набор методов-хелперов, позволяющий подписываться на следующие типы событий:

* DOM-события:
    * `liveBindTo([elemName], event, [callback])` — подписка с отложенной инициализацией на событие на DOM-узле блока или его элементах. Блок будет инициализирован по первому событию `event`. Функция-обработчик `callback` будет вызываться по событию `event` и после инициализации блока.
    * `liveUnbindFrom([elemName], event, [callback])` — удаление подписки с отложенной инициализацией на событие на DOM-узле блока или его элементах.
    * `liveInitOnEvent([elemName], event, callback)` — инициализация по событию на DOM-узле блока или его элементах.
* БЭМ-события:
    * `liveInitOnBlockEvent(event, blockName, callback)` — инициализация по БЭМ-событию экземпляра другого блока, размещенного на DOM-узле текущего экземпляра блока.
    * `liveInitOnBlockInsideEvent(event, blockName, [callback])` — инициализация по БЭМ-событию экземпляра другого блока, вложенного в DOM-узел текущего экземпляра блока.

Например, блок `menu` инициализируется по **БЭМ-событию** `click` вложенного в него блока `menu-item`.

```js
modules.define('menu', ['i-bem__dom', 'menu-item'], function(provide, BEMDOM) {

provide(BEMDOM.decl(this.name, {
    _onItemClick : function(e, data) {
        // функция-обработчик БЭМ-события click на вложенных "menu-item"
    }
}, {
    live : function() {
        this.liveInitOnBlockInsideEvent('click', 'menu-item', function(e, data) {
            this._onItemClick(e, data);
        });
    }
}));

});
```


<a name="init-ajax"></a>
### Инициализация блоков на фрагменте DOM-дерева

Процедура инициализации JS-объектов может быть вызвана
явно для указанного фрагмента DOM-дерева. Часто такая необходимость
возникает при разработке AJAX-интерфейсов, когда нужно [динамически встроить](i-bem-js-dom.ru.md#dynamic) в страницу новые экземпляры блоков либо обновить существующие.

В `i-bem.js` следующие функции выполняют динамическую инициализацию блоков:

* `init`, `destruct` – Инициализация/уничтожение блоков на указанном фрагменте DOM-дерева.
* `update`, `replace`, `append`, `prepend`, `before`, `after` – добавление/замена фрагмента DOM-дерева с одновременной инициализацией блоков на обновленном фрагменте.

Пример использования функций, выполняющих динамическую инициализацию см. в разделе [Динамическое обновление блоков и элементов в DOM-дереве](i-bem-js-dom.ru.md#dynamic).

<a name="destruct-dom"></a>
### Удаление блоков на фрагменте DOM-дерева

Как и процедура инициализации блоков, процедура удаления может быть вызвана явно для заданного фрагмента DOM-дерева. Например, для динамического удаления экземпляров блоков со страницы при разработке AJAX-интерфейсов.

Явный вызов процедуры гарантирует корректное удаление:

* вложенных DOM-узлов;
* блоков, примешанных к другим блокам.

Для явного вызова процедуры удаления служит статический метод `BEMDOM.destruct`.

Метод принимает:

* `ctx` `{jQuery}` – корневой DOM-элемент. Удаляется со всем вложенными DOM-узлами.
* `excludeSelf` `{Boolean}` – не удалять корневой DOM-элемент, если значение `true`. По умолчанию `false`.

<a name="init-bem"></a>
### Инициализация и удаление блоков без DOM-представления

Для создания JS-объектов блока без DOM-представления (не привязанного к HTML-элементу) служит метод `BEM.create`.

Метод принимает:

* `name` `{String|Object}` – имя блока;

Возвращает экземпляр блока указанного класса.

**Удаление** экземпляров блоков без DOM-представления не может быть
выполнено автоматически. Блоки без DOM-представления представляют собой обычные
JS-объекты и удаляются в момент удаления всех ссылок на объект блока.


[i-bem]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[i-bem__dom]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[interact]: ./i-bem-js-interact.ru.md

[params]: ./i-bem-js-params.ru.md
