<a name="ibc"></a>
## Взаимодействие блоков

В рамках БЭМ-методологии блоки следует разрабатывать так, чтобы свести к минимуму зависимость состояний одних блоков от других. Однако на практике идеал полной независимости блоков недостижим.

Взаимодействие блоков может быть реализовано:

* с помощью подписки на [БЭМ-события](i-bem-js-events.ru.md#bem-events) других экземпляров
блоков или подписки на [делегированные БЭМ-события](i-bem-js-events.ru.md#bem-events-delegated);
* с помощью непосредственного вызова методов других экземпляров
блоков или статических методов класса другого блока;
* через проверку состояний одного блока из другого;
* через *каналы событий* (например с помощью элемента [channels](../../common.blocks/events/__channels/events__channels.ru.md) блока `events`).

***

**NB** Не используйте [DOM-события](i-bem-js-events.ru.md#dom-events) для
организации взаимодействия между блоками. DOM-события предназначены
только для реализации внутренних процедур блока.

***

Для реализации взаимодействия блоков `i-bem.js` предоставляет API:

* [Поиск экземпляров блоков в DOM-дереве](i-bem-js-dom.ru.md#api-find)
* [Доступ к экземплярам блоков без DOM-представления](i-bem-js-html-binding.ru.md#api-nodom)
* [Доступ к классам блоков](#api-calss)

<a name="api-class"></a>
### Доступ к классам блоков

JS-компоненты, соответствующие классам блоков, можно получить через [модульную систему][ym]. Это же верно и для блоков [без DOM-представления](i-bem-js-html-binding.ru.md#i-blocks).

Доступ к классам блоков необходим для:

* [Делегирования БЭМ-событий](i-bem-js-events.ru.md#bem-events-delegated).
* [Доопределения](i-bem-js-decl.ru.md#inher-over) декларации блока.
* Вызова статических методов класса.

**Пример:** Вызов статического метода `close` блока `popup` позволяет закрыть все попапы на странице:

```js
modules.define('switcher', ['i-bem__dom', 'popup'], function(provide, BEMDOM, Popup) {

provide(BEMDOM.decl(this.name,
    {
        onSetMod : {
            'popup' : {
                'disabled' : function() {
                    Popup.close();
                }
            }
        }
    }
));

});
```


[ym]: https://github.com/ymaps/modules
