<a name="states"></a>
## Состояния блока

Проектируя динамический блок в стиле БЭМ, нужно представить всю логику
изменений, происходящих в нем, как набор **состояний** блока. Тогда
поведение блока определяется **триггерами** — callback-функциями, которые
выполняются при переходе блока из одного состояния в другое.

Это позволяет писать код блока в декларативном стиле как набор утверждений вида: `описание состояния` — `действия, выполняемые при переходе в данное состояние`.

<a name="modifiers"></a>
### Модификаторы

Согласно БЭМ-методологии, состояние блока и его элементов описывается
**модификаторами**.

Модификатор указывает, в каком из возможных состояний находится блок. Модификатор представляет собой пару: **имя** и **значение**. Список допустимых значений модификатора описывает набор состояний блока. Например, для описания размеров блока
можно использовать модификатор `size` с допустимыми значениями `s`, `m` и
`l`.

**Простой модификатор** — частный случай, когда важно только наличие
или отсутствие модификатора у блока, а его значение несущественно. Например, модификатор, описывающий состояние «отключен»: `disabled`. Модификатор с неуказанным значением `i-bem.js` интерпретирует как булев и автоматически присваивает ему значение `true`.

Каждому блоку можно установить один или несколько модификаторов. Блок
может не иметь модификаторов. Список допустимых модификаторов и их
значений определяет разработчик блока.

Модификаторы устанавливаются при [инициализации экземпляра блока][init] (если модификаторы и их значения указаны в атрибуте `class` соответствующего HTML-элемента).

Модификаторы могут изменяться как в процессе работы блока (например, как реакция на [DOM-события](i-bem-js-events.ru.md#dom-events) блока), там и по запросу из других блоков (см. раздел [Взаимодействие блоков][interact]).

При установке, удалении и изменении значений модификаторов, выполняются [триггеры](#mods-api-trigger).

***

**NB** Если модификаторы были заданы в HTML-элементе блока до момента его инициализации, триггеры на установку данных модификаторов **не выполняются**. Экземпляр блока в этом случае получает начальное состояние, а не меняет его.

***

<a name="mods-api"></a>
#### Управление модификаторами

Методы экземпляра блока для работы с модификаторами:

* `hasMod([elem], modName, [modVal])` – проверяет наличие модификатора. Возвращает `true`, если модификатор `modName` установлен.
* `getMod([elem], modName)` – возвращает значение модификатора `modName`.
* `getMods([elem], [...modNames])` – возвращает хэш со значениями всех модификаторов. Можно получить значения нескольких модификаторов, передав их имена отдельными аргументами (`[...modNames]`). Для получения модификаторов элемента можно указать аргумент `[elem]`.
* `setMod([elem], modName, [modVal=true])` – устанавливает модификатор `modName`. Если значение `modVal` не задано, будет установлен *простой модификатор*.
* `toggleMod([elem], modName, modVal1, [modVal2], [condition])` – переключает значения модификатора. Если передан аргумент `[modVal2]`, переключение происходит между `modVal1` и `modVal2`, если нет, `modVal1` будет поочередно устанавливаться и удаляться. Аргумент `condition` в значении `true` позволяет инвертировать порядок переключения значений модификатора.
* `delMod([elem], modName)` – удаляет модификатор `modName`.

**Пример:**  Метод `changeColor` блока `square` переключает модификатор `color` между значениями `green` и `red`, если у блока выставлен модификатор `has-color`:

```js
BEMDOM.decl('square', {
    changeColor : function(e) {
        if(this.hasMod('has-color')) {
            this.toggleMod('color', 'green', 'red');
        }
    }
});
```


Эти же методы позволяют управлять модификаторами элементов блока. Для этого первым аргументом передается ссылка на **DOM-узел элемента** (а не имя элемента).

**Пример:** Блок `searchbox` по клику может выставлять своему элементу `input` простой модификатор `clean` (подразумеваемое значение — `true`):

```js
BEMDOM.decl('searchbox', {
    _onClick: function() {
        this.setMod(this.elem('input'), 'clean');
    }
});
```


***

**NB** Для изменения значений модификаторов используйте API. Не следует устанавливать модификаторы, самостоятельно изменяя CSS-классы соответствующего DOM-узла.

***

Полное описание API для управления модификаторами приведено в разделе [JSDoc](https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/) блока `i-bem`.

<a name="mods-api-trigger"></a>
### Триггеры на установку модификаторов

Выполнение триггеров на установку модификаторов разбито на две фазы:

1. **До установки модификатора**. Эта фаза зарезервирована для
возможности **отменить** установку модификатора. Если хотя бы один
из триггеров, выполняемых в этой фазе, вернет `false`,
установки модификатора не произойдет.
2. **После установки модификатора**. Триггеры, выполняемые в этой
фазе, уже не могут отменить установку модификаторов.

Триггеры могут быть привязаны к следующим типам изменений значений модификаторов:

1. установка *любого* модификатора в *любое* значение;
2. установка *конкретного* модификатора `modName` в *любое* значение (в том числе
установка простого модификатора в значение `true`);
3. установка *конкретного* модификатора `modName` в *конкретное* значение `modVal`;
4. установка модификатора в значение `''` (пустая строка), что
эквивалентно удалению модификатора или установке простого
модификатора в значение `false`.

При установке модификатора `modName` в значение `modVal` триггеры
каждой фазы (если они определены) вызываются в том порядке, в котором они
перечислены в приведенном выше списке событий (от общего к частному).

Таким образом, при определении триггера пользователь указывает:

* фазу выполнения (до или после установки модификатора);
* тип события (имя и устанавливаемое значение модификатора).

<a name="mods-api-trigger-phase"></a>
#### Фазы выполнения

Дополнительная фаза, предшествующая установке модификатора, позволяет
произвести некоторые проверки без риска повлиять на логику, связанную с установкой модификатора. Например, если существуют взаимоисключающие модификаторы, перед установкой одного из них логично проверить, не установлен ли другой.

**Пример**: модификатор `focused` не будет установлен блоку `searchbox`, если у него есть модификатор `disabled`.

```js
BEMDOM.decl('searchbox', {
    beforeSetMod : {
        'focused' : {
            'true' : function() {
                return !this.hasMod('disabled');
            }
        }
    },

    onSetMod : {
        'focused' : {
            'true' : function() { /* ... */ }
        }
    }
});
```


Если триггер для фазы, предшествующей установке (`beforeSetMod`), возвращает `false`, установка модификатора не производится.

Подробнее об использовании триггеров читайте в разделе [Декларация триггеров](i-bem-js-decl.ru.md#trigger-decl).


***

**NB** Триггер на установку модификатора `js` в значение `inited` является конструктором экземпляра блока, а в значение `''` – деструктором экземпляра блока. Подробности смотрите в разделе [Инициализация][init].

***


[ym]: https://github.com/ymaps/modules

[bem-tools]: https://ru.bem.info/tools/bem/

[i-bem]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[i-bem__dom]: https://ru.bem.info/libs/bem-core/current/desktop/i-bem/jsdoc/

[html]: ./i-bem-js-html-binding.ru.md

[decl]: ./i-bem-js-decl.ru.md

[dom]: ./i-bem-js-dom.ru.md

[states]: ./i-bem-js-states.ru.md

[events]: ./i-bem-js-events.ru.md

[init]: ./i-bem-js-init.ru.md

[interact]: ./i-bem-js-interact.ru.md
